@@ -1,5 +1,3 @@
-{-# LANGUAGE ForeignFunctionInterface #-}
-
 module Termonad.App where
 
 import Termonad.Prelude
@@ -117,6 +115,7 @@ import GI.Vte
   , terminalSearchFindPrevious
   , terminalSearchSetRegex
   , terminalSearchSetWrapAround
+  , terminalSetBoldIsBright
   , terminalSetCursorBlinkMode
   , terminalSetFont
   , terminalSetScrollbackLines
@@ -129,7 +128,8 @@ import Paths_termonad (getDataFileName)
 import Termonad.Gtk (appNew, objFromBuildUnsafe)
 import Termonad.Keys (handleKeyPress)
 import Termonad.Lenses
-  ( lensConfirmExit
+  ( lensBoldIsBright
+  , lensConfirmExit
   , lensCursorBlinkMode
   , lensFontConfig
   , lensOptions
@@ -153,12 +153,15 @@ import Termonad.PreferencesFile (saveToPreferencesFile)
 import Termonad.Term
   ( createTerm
   , relabelTabs
+  , termNextPage
+  , termPrevPage
   , termExitFocused
   , setShowTabs
   , showScrollbarToPolicy
   )
 import Termonad.Types
-  ( FontConfig(..)
+  ( ConfigOptions(..)
+  , FontConfig(..)
   , FontSize(FontSizePoints, FontSizeUnits)
   , ShowScrollbar(..)
   , ShowTabBar(..)
@@ -315,6 +318,7 @@ updateFLTabPos mvarTMState oldPos newPos =
 --
 -- If 'confirmExit' is 'False', then this function always returns
 -- 'ResponseTypeYes'.
+{- HLINT ignore "Reduce duplication" -}
 askShouldExit :: TMState -> IO ResponseType
 askShouldExit mvarTMState = do
   tmState <- readMVar mvarTMState
@@ -427,6 +431,18 @@ setupTermonad tmConfig app win builder = do
   actionMapAddAction app newTabAction
   applicationSetAccelsForAction app "app.newtab" ["<Shift><Ctrl>T"]
 
+  nextPageAction <- simpleActionNew "nextpage" Nothing
+  void $ onSimpleActionActivate nextPageAction $ \_ ->
+    termNextPage mvarTMState
+  actionMapAddAction app nextPageAction
+  applicationSetAccelsForAction app "app.nextpage" ["<Ctrl>Page_Down"]
+
+  prevPageAction <- simpleActionNew "prevpage" Nothing
+  void $ onSimpleActionActivate prevPageAction $ \_ ->
+    termPrevPage mvarTMState
+  actionMapAddAction app prevPageAction
+  applicationSetAccelsForAction app "app.prevpage" ["<Ctrl>Page_Up"]
+
   closeTabAction <- simpleActionNew "closetab" Nothing
   void $ onSimpleActionActivate closeTabAction $ \_ ->
     termExitFocused mvarTMState
@@ -691,9 +707,9 @@ applyNewPreferences mvarTMState = do
   tmState <- readMVar mvarTMState
   let appWin = tmState ^. lensTMStateAppWin
       config = tmState ^. lensTMStateConfig
-      notebook = tmState ^. lensTMStateNotebook ^. lensTMNotebook
-      tabFocusList = tmState ^. lensTMStateNotebook ^. lensTMNotebookTabs
-      showMenu = config  ^. lensOptions ^. lensShowMenu
+      notebook = tmState ^. lensTMStateNotebook . lensTMNotebook
+      tabFocusList = tmState ^. lensTMStateNotebook . lensTMNotebookTabs
+      showMenu = config  ^. lensOptions . lensShowMenu
   applicationWindowSetShowMenubar appWin showMenu
   setShowTabs config notebook
   -- Sets the remaining preferences to each tab
@@ -703,13 +719,15 @@ applyNewPreferencesToTab :: TMState -> TMNotebookTab -> IO ()
 applyNewPreferencesToTab mvarTMState tab = do
   tmState <- readMVar mvarTMState
   let fontDesc = tmState ^. lensTMStateFontDesc
-      term = tab ^. lensTMNotebookTabTerm ^. lensTerm
+      term = tab ^. lensTMNotebookTabTerm . lensTerm
       scrolledWin = tab ^. lensTMNotebookTabTermContainer
-      options = tmState ^. lensTMStateConfig ^. lensOptions
+      options = tmState ^. lensTMStateConfig . lensOptions
   terminalSetFont term (Just fontDesc)
-  terminalSetCursorBlinkMode term (options ^. lensCursorBlinkMode)
-  terminalSetWordCharExceptions term (options ^. lensWordCharExceptions)
-  terminalSetScrollbackLines term (fromIntegral (options ^. lensScrollbackLen))
+  terminalSetCursorBlinkMode term (cursorBlinkMode options)
+  terminalSetWordCharExceptions term (wordCharExceptions options)
+  terminalSetScrollbackLines term (fromIntegral (scrollbackLen options))
+  terminalSetBoldIsBright term (boldIsBright options)
+
   let vScrollbarPolicy = showScrollbarToPolicy (options ^. lensShowScrollbar)
   scrolledWindowSetPolicy scrolledWin PolicyTypeAutomatic vScrollbarPolicy
 
@@ -732,6 +750,8 @@ showPreferencesDialog mvarTMState = do
     objFromBuildUnsafe preferencesBuilder "confirmExit" CheckButton
   showMenuCheckButton <-
     objFromBuildUnsafe preferencesBuilder "showMenu" CheckButton
+  boldIsBrightCheckButton <-
+    objFromBuildUnsafe preferencesBuilder "boldIsBright" CheckButton
   wordCharExceptionsEntryBuffer <-
     objFromBuildUnsafe preferencesBuilder "wordCharExceptions" Entry >>=
       getEntryBuffer
@@ -779,18 +799,14 @@ showPreferencesDialog mvarTMState = do
   -- Init with current state
   fontChooserSetFontDesc fontButton (tmState ^. lensTMStateFontDesc)
   let options = tmState ^. lensTMStateConfig . lensOptions
-  comboBoxSetActive showScrollbarComboBoxText $ options ^. lensShowScrollbar
-  comboBoxSetActive showTabBarComboBoxText $ options ^. lensShowTabBar
-  comboBoxSetActive cursorBlinkModeComboBoxText $ options ^. lensCursorBlinkMode
-  spinButtonSetValue
-    scrollbackLenSpinButton
-    (fromIntegral $ options ^. lensScrollbackLen)
-  toggleButtonSetActive confirmExitCheckButton $ options ^. lensConfirmExit
-  toggleButtonSetActive showMenuCheckButton $ options ^. lensShowMenu
-  entryBufferSetText
-    wordCharExceptionsEntryBuffer
-    (options ^. lensWordCharExceptions)
-    (-1)
+  comboBoxSetActive showScrollbarComboBoxText $ showScrollbar options
+  comboBoxSetActive showTabBarComboBoxText $ showTabBar options
+  comboBoxSetActive cursorBlinkModeComboBoxText $ cursorBlinkMode options
+  spinButtonSetValue scrollbackLenSpinButton (fromIntegral $ scrollbackLen options)
+  toggleButtonSetActive confirmExitCheckButton $ confirmExit options
+  toggleButtonSetActive showMenuCheckButton $ showMenu options
+  toggleButtonSetActive boldIsBrightCheckButton $ boldIsBright options
+  entryBufferSetText wordCharExceptionsEntryBuffer (wordCharExceptions options) (-1)
 
   -- Run dialog then close
   res <- dialogRun preferencesDialog
@@ -799,28 +815,30 @@ showPreferencesDialog mvarTMState = do
   when (toEnum (fromIntegral res) == ResponseTypeAccept) $ do
     maybeFontDesc <- fontChooserGetFontDesc fontButton
     maybeFontConfig <-
-      liftM join $ mapM fontConfigFromFontDescription maybeFontDesc
+      join <$> mapM fontConfigFromFontDescription maybeFontDesc
     maybeShowScrollbar <-
       comboBoxGetActive showScrollbarComboBoxText [ShowScrollbarNever ..]
     maybeShowTabBar <-
       comboBoxGetActive showTabBarComboBoxText [ShowTabBarNever ..]
     maybeCursorBlinkMode <-
       comboBoxGetActive cursorBlinkModeComboBoxText [CursorBlinkModeSystem ..]
-    scrollbackLen <-
+    scrollbackLenVal <-
       fromIntegral <$> spinButtonGetValueAsInt scrollbackLenSpinButton
-    confirmExit <- toggleButtonGetActive confirmExitCheckButton
-    showMenu <- toggleButtonGetActive showMenuCheckButton
-    wordCharExceptions <- entryBufferGetText wordCharExceptionsEntryBuffer
+    confirmExitVal <- toggleButtonGetActive confirmExitCheckButton
+    showMenuVal <- toggleButtonGetActive showMenuCheckButton
+    boldIsBrightVal <- toggleButtonGetActive boldIsBrightCheckButton
+    wordCharExceptionsVal <- entryBufferGetText wordCharExceptionsEntryBuffer
 
     -- Apply the changes to mvarTMState
     modifyMVar_ mvarTMState $ pure
       . over lensTMStateFontDesc (`fromMaybe` maybeFontDesc)
       . over (lensTMStateConfig . lensOptions)
-        ( set lensConfirmExit confirmExit
-        . set lensShowMenu showMenu
-        . set lensWordCharExceptions wordCharExceptions
+        ( set lensConfirmExit confirmExitVal
+        . set lensShowMenu showMenuVal
+        . set lensBoldIsBright boldIsBrightVal
+        . set lensWordCharExceptions wordCharExceptionsVal
         . over lensFontConfig (`fromMaybe` maybeFontConfig)
-        . set lensScrollbackLen scrollbackLen
+        . set lensScrollbackLen scrollbackLenVal
         . over lensShowScrollbar (`fromMaybe` maybeShowScrollbar)
         . over lensShowTabBar (`fromMaybe` maybeShowTabBar)
         . over lensCursorBlinkMode (`fromMaybe` maybeCursorBlinkMode)
@@ -923,7 +941,7 @@ defaultMain tmConfig = do
             "configuration file and running termonad with default settings."
           start tmConfig
       | otherwise -> do
-          putStrLn $ "IO error occurred when trying to run termonad:"
+          putStrLn "IO error occurred when trying to run termonad:"
           print ioErr
           putStrLn "Don't know how to recover.  Exiting."
     Right _ -> pure ()
