@@ -59,7 +59,9 @@ import GI.Gtk
   , notebookAppendPage
   , notebookDetachTab
   , notebookGetNPages
+  , notebookNextPage
   , notebookPageNum
+  , notebookPrevPage
   , notebookSetCurrentPage
   , notebookSetShowTabs
   , notebookSetTabReorderable
@@ -91,6 +93,7 @@ import GI.Vte
   , terminalSetScrollbackLines
   , terminalSpawnSync
   , terminalSetWordCharExceptions
+  , terminalSetBoldIsBright
   )
 import System.Directory (getSymbolicLinkTarget)
 import System.Environment (lookupEnv)
@@ -111,7 +114,7 @@ import Termonad.Lenses
   )
 import Termonad.Types
   ( ConfigHooks(createTermHook)
-  , ConfigOptions(scrollbackLen, wordCharExceptions, cursorBlinkMode)
+  , ConfigOptions(scrollbackLen, wordCharExceptions, cursorBlinkMode, boldIsBright)
   , ShowScrollbar(..)
   , ShowTabBar(..)
   , TMConfig(hooks, options)
@@ -138,6 +141,16 @@ focusTerm i mvarTMState = do
 altNumSwitchTerm :: Int -> TMState -> IO ()
 altNumSwitchTerm = focusTerm
 
+termNextPage :: TMState -> IO ()
+termNextPage mvarTMState = do
+  note <- tmNotebook . tmStateNotebook <$> readMVar mvarTMState
+  notebookNextPage note
+
+termPrevPage :: TMState -> IO ()
+termPrevPage mvarTMState = do
+  note <- tmNotebook . tmStateNotebook <$> readMVar mvarTMState
+  notebookPrevPage note
+
 termExitFocused :: TMState -> IO ()
 termExitFocused mvarTMState = do
   tmState <- readMVar mvarTMState
@@ -346,6 +359,7 @@ createAndInitVteTerm tmStateFontDesc curOpts = do
   terminalSetWordCharExceptions vteTerm $ wordCharExceptions curOpts
   terminalSetScrollbackLines vteTerm (fromIntegral (scrollbackLen curOpts))
   terminalSetCursorBlinkMode vteTerm (cursorBlinkMode curOpts)
+  terminalSetBoldIsBright vteTerm (boldIsBright curOpts)
   widgetShow vteTerm
   pure vteTerm
 
@@ -362,7 +376,7 @@ launchShell vteTerm maybeCurrDir = do
   -- Should probably use GI.Vte.Functions.getUserShell, but contrary to its
   -- documentation it raises an exception rather wrap in Maybe.
   mShell <- lookupEnv "SHELL"
-  let argv = fromMaybe ["/usr/bin/env", "bash"] (pure <$> mShell)
+  let argv = maybe ["/usr/bin/env", "bash"] pure mShell
   -- Launch the shell
   shellPid <-
     terminalSpawnSync
@@ -479,8 +493,8 @@ handleMousePress vteTerm event = do
     menuModel <- menuNew
     menuAppend menuModel (Just "Copy") (Just "app.copy")
     menuAppend menuModel (Just "Paste") (Just "app.paste")
+    menuAppend menuModel (Just "Preferences") (Just "app.preferences")
     menu <- menuNewFromModel menuModel
     menuAttachToWidget menu vteTerm Nothing
     menuPopupAtPointer menu Nothing
   pure rightClick
-
